import { Injectable } from '@angular/core';
import { Observable ,  Subject } from 'rxjs';
import { map } from 'rxjs/operators';
import { HttpClient } from '@angular/common/http';
import { Resource, IResource, ResourceRO } from '../models/resource.model';
import { environment } from 'src/environments/environment';

@Injectable()
export class ResourceService {

  limit: number;
  pageNumber: number;
  sortOptions: {
    value: string,
    order: string,
  };

  private _searchChangeSubject: Subject<any>;

  constructor(
    private _httpClient: HttpClient) {
      this._searchChangeSubject = new Subject<any>();
      this.limit = 10;
      this.pageNumber = 1;
  }

  getResources(options?): Observable<ResourceRO> {
    let query = '?';
    query += 'limit=' + (this.limit ? this.limit : 20);
    query += '&offset=' + (this.pageNumber ? (this.pageNumber - 1) * this.limit : 0);
    query += '&sort_by=' + this.sortOptions.value + '.' + this.sortOptions.order;

    return this._httpClient.get<IResource[]>(environment.resources.url + query, { observe: 'response' }).pipe(
      map((response) => {
        const totalCount = response.headers.get('Content-Range');
        const resources = [];
        response.body.forEach((resource) => {
          resources.push(new Resource(resource));
        });
        return new ResourceRO(resources, parseInt(totalCount, 10));
      }));
  }

  findById(id): Observable<Resource> {
    return this._httpClient.get<IResource>(environment.resources.url + id).pipe(
      map((response) => {
        return new Resource(response);
      }
      ));
  }

  delete(id) {
    return this._httpClient.delete(environment.resources.url + id);
  }

  replaceOrCreate(data): Observable<Resource> {
    if (data.id) {
      return this._httpClient.put<IResource>(environment.resources.url + data.id, data).pipe(
        map((response) => {
          return new Resource(response);
        }
        ));
    }
    return this._httpClient.post<IResource>(environment.resources.url, data).pipe(
      map((response) => {
        return new Resource(response);
      }
      ));
  }

    /* PAGINATION */
    paginationChanged(limit: number, pageNumber: number) {
      this.limit = limit;
      this.pageNumber = pageNumber;
      this._searchChangeSubject.next();
    }

    reverseSortOrder(): void {
      if (this.sortOptions.order === 'asc') {
        this.sortOptions.order = 'desc';
      } else {
        this.sortOptions.order = 'asc';
      }
    }

    get searchChange$(): Observable<string> {
      return this._searchChangeSubject.asObservable();
    }
}
